# -*- coding: utf-8 -*-
#---------------------------------------------------------------------------
# Copyright 2023 VMware, Inc.  All rights reserved.

# AUTO GENERATED FILE -- DO NOT MODIFY!
#
# vAPI stub file for package com.vmware.nsx.logical_ports.
#---------------------------------------------------------------------------

"""


"""

__author__ = 'VMware, Inc.'
__docformat__ = 'restructuredtext en'

import sys
from warnings import warn

from vmware.vapi.bindings import type
from vmware.vapi.bindings.converter import TypeConverter
from vmware.vapi.bindings.enum import Enum
from vmware.vapi.bindings.error import VapiError
from vmware.vapi.bindings.struct import VapiStruct
from vmware.vapi.bindings.stub import (
    ApiInterfaceStub, StubFactoryBase, VapiInterface)
from vmware.vapi.bindings.common import raise_core_exception
from vmware.vapi.data.validator import (UnionValidator, HasFieldsOfValidator)
from vmware.vapi.exception import CoreException
from vmware.vapi.lib.constants import TaskType
from vmware.vapi.lib.rest import OperationRestMetadata


class ForwardingPath(VapiInterface):
    """
    
    """

    _VAPI_SERVICE_ID = 'com.vmware.nsx.logical_ports.forwarding_path'
    """
    Identifier of the service in canonical form.
    """
    def __init__(self, config):
        """
        :type  config: :class:`vmware.vapi.bindings.stub.StubConfiguration`
        :param config: Configuration to be used for creating the stub.
        """
        VapiInterface.__init__(self, config, _ForwardingPathStub)
        self._VAPI_OPERATION_IDS = {}


    def get(self,
            lport_id,
            peer_port_id,
            ):
        """
        Get networking entities between two logical ports with VIF attachment

        :type  lport_id: :class:`str`
        :param lport_id: ID of source port (required)
        :type  peer_port_id: :class:`str`
        :param peer_port_id: ID of peer port (required)
        :rtype: :class:`com.vmware.nsx.model_client.PortConnectionEntities`
        :return: com.vmware.nsx.model.PortConnectionEntities
        :raise: :class:`com.vmware.vapi.std.errors_client.InvalidRequest` 
            Bad Request, Precondition Failed
        :raise: :class:`com.vmware.vapi.std.errors_client.Unauthorized` 
            Forbidden
        :raise: :class:`com.vmware.vapi.std.errors_client.ServiceUnavailable` 
            Service Unavailable
        :raise: :class:`com.vmware.vapi.std.errors_client.InternalServerError` 
            Internal Server Error
        :raise: :class:`com.vmware.vapi.std.errors_client.NotFound` 
            Not Found
        """
        return self._invoke('get',
                            {
                            'lport_id': lport_id,
                            'peer_port_id': peer_port_id,
                            })
class MacTable(VapiInterface):
    """
    
    """
    LIST_SOURCE_REALTIME = "realtime"
    """
    Possible value for ``source`` of method :func:`MacTable.list`.

    """
    LIST_SOURCE_CACHED = "cached"
    """
    Possible value for ``source`` of method :func:`MacTable.list`.

    """

    _VAPI_SERVICE_ID = 'com.vmware.nsx.logical_ports.mac_table'
    """
    Identifier of the service in canonical form.
    """
    def __init__(self, config):
        """
        :type  config: :class:`vmware.vapi.bindings.stub.StubConfiguration`
        :param config: Configuration to be used for creating the stub.
        """
        VapiInterface.__init__(self, config, _MacTableStub)
        self._VAPI_OPERATION_IDS = {}


    def list(self,
             lport_id,
             cursor=None,
             included_fields=None,
             page_size=None,
             sort_ascending=None,
             sort_by=None,
             source=None,
             transport_node_id=None,
             ):
        """
        Returns MAC table of a specified logical port. If the target transport
        node id is not provided, the NSX manager will ask the controller for
        the transport node where the logical port is located. The query
        parameter \"source=cached\" is not supported. MAC table retrieval is
        not supported on logical ports that are attached to a logical router. 
        
        This api is deprecated from 3.2.2. Please use policy api - 
        /infra/segments/<segment-id>/ports/<segment-port-id>/mac-table

        .. deprecated:: unknown

        :type  lport_id: :class:`str`
        :param lport_id: (required)
        :type  cursor: :class:`str` or ``None``
        :param cursor: Opaque cursor to be used for getting next page of records (supplied
            by current result page) (optional)
        :type  included_fields: :class:`str` or ``None``
        :param included_fields: Comma separated list of fields that should be included in query
            result (optional)
        :type  page_size: :class:`long` or ``None``
        :param page_size: Maximum number of results to return in this page (server may return
            fewer) (optional, default to 1000)
        :type  sort_ascending: :class:`bool` or ``None``
        :param sort_ascending: (optional)
        :type  sort_by: :class:`str` or ``None``
        :param sort_by: Field by which records are sorted (optional)
        :type  source: :class:`str` or ``None``
        :param source: Data source type. (optional)
        :type  transport_node_id: :class:`str` or ``None``
        :param transport_node_id: TransportNode Id (optional)
        :rtype: :class:`com.vmware.nsx.model_client.LogicalPortMacAddressListResult`
        :return: com.vmware.nsx.model.LogicalPortMacAddressListResult
        :raise: :class:`com.vmware.vapi.std.errors_client.InvalidRequest` 
            Bad Request, Precondition Failed
        :raise: :class:`com.vmware.vapi.std.errors_client.Unauthorized` 
            Forbidden
        :raise: :class:`com.vmware.vapi.std.errors_client.ServiceUnavailable` 
            Service Unavailable
        :raise: :class:`com.vmware.vapi.std.errors_client.InternalServerError` 
            Internal Server Error
        :raise: :class:`com.vmware.vapi.std.errors_client.NotFound` 
            Not Found
        """
        warn('com.vmware.nsx.logical_ports.MacTable.list is deprecated.', DeprecationWarning)
        return self._invoke('list',
                            {
                            'lport_id': lport_id,
                            'cursor': cursor,
                            'included_fields': included_fields,
                            'page_size': page_size,
                            'sort_ascending': sort_ascending,
                            'sort_by': sort_by,
                            'source': source,
                            'transport_node_id': transport_node_id,
                            })
class State(VapiInterface):
    """
    
    """

    _VAPI_SERVICE_ID = 'com.vmware.nsx.logical_ports.state'
    """
    Identifier of the service in canonical form.
    """
    def __init__(self, config):
        """
        :type  config: :class:`vmware.vapi.bindings.stub.StubConfiguration`
        :param config: Configuration to be used for creating the stub.
        """
        VapiInterface.__init__(self, config, _StateStub)
        self._VAPI_OPERATION_IDS = {}


    def get(self,
            lport_id,
            ):
        """
        Returns transport node id for a specified logical port. Also returns
        information about all address bindings of the specified logical port.
        This includes address bindings discovered via various snooping methods
        like ARP snooping, DHCP snooping etc. and addressing bindings that are
        realized based on user configuration. 
        This api is now deprecated. Please use new api - GET
        policy/api/v1/infra/realized-state/realized-entities?intent_path={{intent-path}}
        GET
        policy/api/v1/infra/realized-state/status?intent_path={{intent-path}}

        .. deprecated:: unknown

        :type  lport_id: :class:`str`
        :param lport_id: (required)
        :rtype: :class:`com.vmware.nsx.model_client.LogicalPortState`
        :return: com.vmware.nsx.model.LogicalPortState
        :raise: :class:`com.vmware.vapi.std.errors_client.InvalidRequest` 
            Bad Request, Precondition Failed
        :raise: :class:`com.vmware.vapi.std.errors_client.Unauthorized` 
            Forbidden
        :raise: :class:`com.vmware.vapi.std.errors_client.ServiceUnavailable` 
            Service Unavailable
        :raise: :class:`com.vmware.vapi.std.errors_client.InternalServerError` 
            Internal Server Error
        :raise: :class:`com.vmware.vapi.std.errors_client.NotFound` 
            Not Found
        """
        warn('com.vmware.nsx.logical_ports.State.get is deprecated.', DeprecationWarning)
        return self._invoke('get',
                            {
                            'lport_id': lport_id,
                            })
class Statistics(VapiInterface):
    """
    
    """
    GET_SOURCE_REALTIME = "realtime"
    """
    Possible value for ``source`` of method :func:`Statistics.get`.

    """
    GET_SOURCE_CACHED = "cached"
    """
    Possible value for ``source`` of method :func:`Statistics.get`.

    """

    _VAPI_SERVICE_ID = 'com.vmware.nsx.logical_ports.statistics'
    """
    Identifier of the service in canonical form.
    """
    def __init__(self, config):
        """
        :type  config: :class:`vmware.vapi.bindings.stub.StubConfiguration`
        :param config: Configuration to be used for creating the stub.
        """
        VapiInterface.__init__(self, config, _StatisticsStub)
        self._VAPI_OPERATION_IDS = {}


    def get(self,
            lport_id,
            source=None,
            ):
        """
        Returns statistics of a specified logical port. If the logical port is
        attached to a logical router port, query parameter \"source=realtime\"
        is not supported. 
        
        This api is deprecated from 3.2.2. Please use policy api - 
        /infra/segments/<segment-id>/ports/<segment-port-id>/statistics

        .. deprecated:: unknown

        :type  lport_id: :class:`str`
        :param lport_id: (required)
        :type  source: :class:`str` or ``None``
        :param source: Data source type. (optional)
        :rtype: :class:`com.vmware.nsx.model_client.LogicalPortStatistics`
        :return: com.vmware.nsx.model.LogicalPortStatistics
        :raise: :class:`com.vmware.vapi.std.errors_client.InvalidRequest` 
            Bad Request, Precondition Failed
        :raise: :class:`com.vmware.vapi.std.errors_client.Unauthorized` 
            Forbidden
        :raise: :class:`com.vmware.vapi.std.errors_client.ServiceUnavailable` 
            Service Unavailable
        :raise: :class:`com.vmware.vapi.std.errors_client.InternalServerError` 
            Internal Server Error
        :raise: :class:`com.vmware.vapi.std.errors_client.NotFound` 
            Not Found
        """
        warn('com.vmware.nsx.logical_ports.Statistics.get is deprecated.', DeprecationWarning)
        return self._invoke('get',
                            {
                            'lport_id': lport_id,
                            'source': source,
                            })
class Status(VapiInterface):
    """
    
    """
    GET_SOURCE_REALTIME = "realtime"
    """
    Possible value for ``source`` of method :func:`Status.get`.

    """
    GET_SOURCE_CACHED = "cached"
    """
    Possible value for ``source`` of method :func:`Status.get`.

    """
    GETALL_ATTACHMENT_TYPE_VIF = "VIF"
    """
    Possible value for ``attachmentType`` of method :func:`Status.getall`.

    """
    GETALL_ATTACHMENT_TYPE_LOGICALROUTER = "LOGICALROUTER"
    """
    Possible value for ``attachmentType`` of method :func:`Status.getall`.

    """
    GETALL_ATTACHMENT_TYPE_BRIDGEENDPOINT = "BRIDGEENDPOINT"
    """
    Possible value for ``attachmentType`` of method :func:`Status.getall`.

    """
    GETALL_ATTACHMENT_TYPE_DHCP_SERVICE = "DHCP_SERVICE"
    """
    Possible value for ``attachmentType`` of method :func:`Status.getall`.

    """
    GETALL_ATTACHMENT_TYPE_METADATA_PROXY = "METADATA_PROXY"
    """
    Possible value for ``attachmentType`` of method :func:`Status.getall`.

    """
    GETALL_ATTACHMENT_TYPE_L2VPN_SESSION = "L2VPN_SESSION"
    """
    Possible value for ``attachmentType`` of method :func:`Status.getall`.

    """
    GETALL_ATTACHMENT_TYPE_NONE = "NONE"
    """
    Possible value for ``attachmentType`` of method :func:`Status.getall`.

    """
    GETALL_SOURCE_REALTIME = "realtime"
    """
    Possible value for ``source`` of method :func:`Status.getall`.

    """
    GETALL_SOURCE_CACHED = "cached"
    """
    Possible value for ``source`` of method :func:`Status.getall`.

    """

    _VAPI_SERVICE_ID = 'com.vmware.nsx.logical_ports.status'
    """
    Identifier of the service in canonical form.
    """
    def __init__(self, config):
        """
        :type  config: :class:`vmware.vapi.bindings.stub.StubConfiguration`
        :param config: Configuration to be used for creating the stub.
        """
        VapiInterface.__init__(self, config, _StatusStub)
        self._VAPI_OPERATION_IDS = {}


    def get(self,
            lport_id,
            source=None,
            ):
        """
        Returns operational status of a specified logical port. 
        
        This api is deprecated from 3.2.2. Please use policy api - 
        /infra/segments/<segment-id>/ports/<segment-port-id>/status

        .. deprecated:: unknown

        :type  lport_id: :class:`str`
        :param lport_id: (required)
        :type  source: :class:`str` or ``None``
        :param source: Data source type. (optional)
        :rtype: :class:`com.vmware.nsx.model_client.LogicalPortOperationalStatus`
        :return: com.vmware.nsx.model.LogicalPortOperationalStatus
        :raise: :class:`com.vmware.vapi.std.errors_client.InvalidRequest` 
            Bad Request, Precondition Failed
        :raise: :class:`com.vmware.vapi.std.errors_client.Unauthorized` 
            Forbidden
        :raise: :class:`com.vmware.vapi.std.errors_client.ServiceUnavailable` 
            Service Unavailable
        :raise: :class:`com.vmware.vapi.std.errors_client.InternalServerError` 
            Internal Server Error
        :raise: :class:`com.vmware.vapi.std.errors_client.NotFound` 
            Not Found
        """
        warn('com.vmware.nsx.logical_ports.Status.get is deprecated.', DeprecationWarning)
        return self._invoke('get',
                            {
                            'lport_id': lport_id,
                            'source': source,
                            })

    def getall(self,
               attachment_id=None,
               attachment_type=None,
               bridge_cluster_id=None,
               container_ports_only=None,
               cursor=None,
               diagnostic=None,
               included_fields=None,
               logical_switch_id=None,
               page_size=None,
               parent_vif_id=None,
               sort_ascending=None,
               sort_by=None,
               source=None,
               switching_profile_id=None,
               transport_node_id=None,
               transport_zone_id=None,
               ):
        """
        Returns operational status of all logical ports. The query parameter
        \"source=realtime\" is not supported. Pagination is not supported for
        this API. The query parameters \"cursor\", \"sort_ascending\",
        \"sort_by\", \"page_size\" and \"included_fields\" will be ignored. 
        
        This api is deprecated from 3.2.2. Please use policy api - 
        /search/query?query=resource_type:SegmentPort&included_fields=id&included_fields=admin_state

        .. deprecated:: unknown

        :type  attachment_id: :class:`str` or ``None``
        :param attachment_id: Logical Port attachment Id (optional)
        :type  attachment_type: :class:`str` or ``None``
        :param attachment_type: Type of attachment for logical port; for query only. (optional)
        :type  bridge_cluster_id: :class:`str` or ``None``
        :param bridge_cluster_id: Bridge Cluster identifier (optional)
        :type  container_ports_only: :class:`bool` or ``None``
        :param container_ports_only: Only container VIF logical ports will be returned if true
            (optional, default to false)
        :type  cursor: :class:`str` or ``None``
        :param cursor: Opaque cursor to be used for getting next page of records (supplied
            by current result page) (optional)
        :type  diagnostic: :class:`bool` or ``None``
        :param diagnostic: Flag to enable showing of transit logical port. (optional, default
            to false)
        :type  included_fields: :class:`str` or ``None``
        :param included_fields: Comma separated list of fields that should be included in query
            result (optional)
        :type  logical_switch_id: :class:`str` or ``None``
        :param logical_switch_id: Logical Switch identifier (optional)
        :type  page_size: :class:`long` or ``None``
        :param page_size: Maximum number of results to return in this page (server may return
            fewer) (optional, default to 1000)
        :type  parent_vif_id: :class:`str` or ``None``
        :param parent_vif_id: ID of the VIF of type PARENT (optional)
        :type  sort_ascending: :class:`bool` or ``None``
        :param sort_ascending: (optional)
        :type  sort_by: :class:`str` or ``None``
        :param sort_by: Field by which records are sorted (optional)
        :type  source: :class:`str` or ``None``
        :param source: Data source type. (optional)
        :type  switching_profile_id: :class:`str` or ``None``
        :param switching_profile_id: Network Profile identifier (optional)
        :type  transport_node_id: :class:`str` or ``None``
        :param transport_node_id: Transport node identifier (optional)
        :type  transport_zone_id: :class:`str` or ``None``
        :param transport_zone_id: Transport zone identifier (optional)
        :rtype: :class:`com.vmware.nsx.model_client.LogicalPortStatusSummary`
        :return: com.vmware.nsx.model.LogicalPortStatusSummary
        :raise: :class:`com.vmware.vapi.std.errors_client.InvalidRequest` 
            Bad Request, Precondition Failed
        :raise: :class:`com.vmware.vapi.std.errors_client.Unauthorized` 
            Forbidden
        :raise: :class:`com.vmware.vapi.std.errors_client.ServiceUnavailable` 
            Service Unavailable
        :raise: :class:`com.vmware.vapi.std.errors_client.InternalServerError` 
            Internal Server Error
        :raise: :class:`com.vmware.vapi.std.errors_client.NotFound` 
            Not Found
        """
        warn('com.vmware.nsx.logical_ports.Status.getall is deprecated.', DeprecationWarning)
        return self._invoke('getall',
                            {
                            'attachment_id': attachment_id,
                            'attachment_type': attachment_type,
                            'bridge_cluster_id': bridge_cluster_id,
                            'container_ports_only': container_ports_only,
                            'cursor': cursor,
                            'diagnostic': diagnostic,
                            'included_fields': included_fields,
                            'logical_switch_id': logical_switch_id,
                            'page_size': page_size,
                            'parent_vif_id': parent_vif_id,
                            'sort_ascending': sort_ascending,
                            'sort_by': sort_by,
                            'source': source,
                            'switching_profile_id': switching_profile_id,
                            'transport_node_id': transport_node_id,
                            'transport_zone_id': transport_zone_id,
                            })
class _ForwardingPathStub(ApiInterfaceStub):
    def __init__(self, config):
        # properties for get operation
        get_input_type = type.StructType('operation-input', {
            'lport_id': type.StringType(),
            'peer_port_id': type.StringType(),
        })
        get_error_dict = {
            'com.vmware.vapi.std.errors.invalid_request':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'InvalidRequest'),
            'com.vmware.vapi.std.errors.unauthorized':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'Unauthorized'),
            'com.vmware.vapi.std.errors.service_unavailable':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'ServiceUnavailable'),
            'com.vmware.vapi.std.errors.internal_server_error':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'InternalServerError'),
            'com.vmware.vapi.std.errors.not_found':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'NotFound'),

        }
        get_input_value_validator_list = [
        ]
        get_output_validator_list = [
            HasFieldsOfValidator()
        ]
        get_rest_metadata = OperationRestMetadata(
            http_method='GET',
            url_template='/api/v1/logical-ports/{lport-id}/forwarding-path',
            path_variables={
                'lport_id': 'lport-id',
            },
             header_parameters={
                 },
            query_parameters={
                'peer_port_id': 'peer_port_id',
            },
            content_type='application/json'
        )

        operations = {
            'get': {
                'input_type': get_input_type,
                'output_type': type.ReferenceType('com.vmware.nsx.model_client', 'PortConnectionEntities'),
                'errors': get_error_dict,
                'input_value_validator_list': get_input_value_validator_list,
                'output_validator_list': get_output_validator_list,
                'task_type': TaskType.NONE,
            },
        }
        rest_metadata = {
            'get': get_rest_metadata,
        }
        ApiInterfaceStub.__init__(
            self, iface_name='com.vmware.nsx.logical_ports.forwarding_path',
            config=config, operations=operations, rest_metadata=rest_metadata,
            is_vapi_rest=False)

class _MacTableStub(ApiInterfaceStub):
    def __init__(self, config):
        # properties for list operation
        list_input_type = type.StructType('operation-input', {
            'lport_id': type.StringType(),
            'cursor': type.OptionalType(type.StringType()),
            'included_fields': type.OptionalType(type.StringType()),
            'page_size': type.OptionalType(type.IntegerType()),
            'sort_ascending': type.OptionalType(type.BooleanType()),
            'sort_by': type.OptionalType(type.StringType()),
            'source': type.OptionalType(type.StringType()),
            'transport_node_id': type.OptionalType(type.StringType()),
        })
        list_error_dict = {
            'com.vmware.vapi.std.errors.invalid_request':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'InvalidRequest'),
            'com.vmware.vapi.std.errors.unauthorized':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'Unauthorized'),
            'com.vmware.vapi.std.errors.service_unavailable':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'ServiceUnavailable'),
            'com.vmware.vapi.std.errors.internal_server_error':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'InternalServerError'),
            'com.vmware.vapi.std.errors.not_found':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'NotFound'),

        }
        list_input_value_validator_list = [
        ]
        list_output_validator_list = [
        ]
        list_rest_metadata = OperationRestMetadata(
            http_method='GET',
            url_template='/api/v1/logical-ports/{lport-id}/mac-table',
            path_variables={
                'lport_id': 'lport-id',
            },
             header_parameters={
                             },
            query_parameters={
                'cursor': 'cursor',
                'included_fields': 'included_fields',
                'page_size': 'page_size',
                'sort_ascending': 'sort_ascending',
                'sort_by': 'sort_by',
                'source': 'source',
                'transport_node_id': 'transport_node_id',
            },
            content_type='application/json'
        )

        operations = {
            'list': {
                'input_type': list_input_type,
                'output_type': type.ReferenceType('com.vmware.nsx.model_client', 'LogicalPortMacAddressListResult'),
                'errors': list_error_dict,
                'input_value_validator_list': list_input_value_validator_list,
                'output_validator_list': list_output_validator_list,
                'task_type': TaskType.NONE,
            },
        }
        rest_metadata = {
            'list': list_rest_metadata,
        }
        ApiInterfaceStub.__init__(
            self, iface_name='com.vmware.nsx.logical_ports.mac_table',
            config=config, operations=operations, rest_metadata=rest_metadata,
            is_vapi_rest=False)

class _StateStub(ApiInterfaceStub):
    def __init__(self, config):
        # properties for get operation
        get_input_type = type.StructType('operation-input', {
            'lport_id': type.StringType(),
        })
        get_error_dict = {
            'com.vmware.vapi.std.errors.invalid_request':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'InvalidRequest'),
            'com.vmware.vapi.std.errors.unauthorized':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'Unauthorized'),
            'com.vmware.vapi.std.errors.service_unavailable':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'ServiceUnavailable'),
            'com.vmware.vapi.std.errors.internal_server_error':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'InternalServerError'),
            'com.vmware.vapi.std.errors.not_found':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'NotFound'),

        }
        get_input_value_validator_list = [
        ]
        get_output_validator_list = [
        ]
        get_rest_metadata = OperationRestMetadata(
            http_method='GET',
            url_template='/api/v1/logical-ports/{lport-id}/state',
            path_variables={
                'lport_id': 'lport-id',
            },
             header_parameters={
               },
            query_parameters={
            },
            content_type='application/json'
        )

        operations = {
            'get': {
                'input_type': get_input_type,
                'output_type': type.ReferenceType('com.vmware.nsx.model_client', 'LogicalPortState'),
                'errors': get_error_dict,
                'input_value_validator_list': get_input_value_validator_list,
                'output_validator_list': get_output_validator_list,
                'task_type': TaskType.NONE,
            },
        }
        rest_metadata = {
            'get': get_rest_metadata,
        }
        ApiInterfaceStub.__init__(
            self, iface_name='com.vmware.nsx.logical_ports.state',
            config=config, operations=operations, rest_metadata=rest_metadata,
            is_vapi_rest=False)

class _StatisticsStub(ApiInterfaceStub):
    def __init__(self, config):
        # properties for get operation
        get_input_type = type.StructType('operation-input', {
            'lport_id': type.StringType(),
            'source': type.OptionalType(type.StringType()),
        })
        get_error_dict = {
            'com.vmware.vapi.std.errors.invalid_request':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'InvalidRequest'),
            'com.vmware.vapi.std.errors.unauthorized':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'Unauthorized'),
            'com.vmware.vapi.std.errors.service_unavailable':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'ServiceUnavailable'),
            'com.vmware.vapi.std.errors.internal_server_error':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'InternalServerError'),
            'com.vmware.vapi.std.errors.not_found':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'NotFound'),

        }
        get_input_value_validator_list = [
        ]
        get_output_validator_list = [
        ]
        get_rest_metadata = OperationRestMetadata(
            http_method='GET',
            url_template='/api/v1/logical-ports/{lport-id}/statistics',
            path_variables={
                'lport_id': 'lport-id',
            },
             header_parameters={
                 },
            query_parameters={
                'source': 'source',
            },
            content_type='application/json'
        )

        operations = {
            'get': {
                'input_type': get_input_type,
                'output_type': type.ReferenceType('com.vmware.nsx.model_client', 'LogicalPortStatistics'),
                'errors': get_error_dict,
                'input_value_validator_list': get_input_value_validator_list,
                'output_validator_list': get_output_validator_list,
                'task_type': TaskType.NONE,
            },
        }
        rest_metadata = {
            'get': get_rest_metadata,
        }
        ApiInterfaceStub.__init__(
            self, iface_name='com.vmware.nsx.logical_ports.statistics',
            config=config, operations=operations, rest_metadata=rest_metadata,
            is_vapi_rest=False)

class _StatusStub(ApiInterfaceStub):
    def __init__(self, config):
        # properties for get operation
        get_input_type = type.StructType('operation-input', {
            'lport_id': type.StringType(),
            'source': type.OptionalType(type.StringType()),
        })
        get_error_dict = {
            'com.vmware.vapi.std.errors.invalid_request':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'InvalidRequest'),
            'com.vmware.vapi.std.errors.unauthorized':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'Unauthorized'),
            'com.vmware.vapi.std.errors.service_unavailable':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'ServiceUnavailable'),
            'com.vmware.vapi.std.errors.internal_server_error':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'InternalServerError'),
            'com.vmware.vapi.std.errors.not_found':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'NotFound'),

        }
        get_input_value_validator_list = [
        ]
        get_output_validator_list = [
        ]
        get_rest_metadata = OperationRestMetadata(
            http_method='GET',
            url_template='/api/v1/logical-ports/{lport-id}/status',
            path_variables={
                'lport_id': 'lport-id',
            },
             header_parameters={
                 },
            query_parameters={
                'source': 'source',
            },
            content_type='application/json'
        )

        # properties for getall operation
        getall_input_type = type.StructType('operation-input', {
            'attachment_id': type.OptionalType(type.StringType()),
            'attachment_type': type.OptionalType(type.StringType()),
            'bridge_cluster_id': type.OptionalType(type.StringType()),
            'container_ports_only': type.OptionalType(type.BooleanType()),
            'cursor': type.OptionalType(type.StringType()),
            'diagnostic': type.OptionalType(type.BooleanType()),
            'included_fields': type.OptionalType(type.StringType()),
            'logical_switch_id': type.OptionalType(type.StringType()),
            'page_size': type.OptionalType(type.IntegerType()),
            'parent_vif_id': type.OptionalType(type.StringType()),
            'sort_ascending': type.OptionalType(type.BooleanType()),
            'sort_by': type.OptionalType(type.StringType()),
            'source': type.OptionalType(type.StringType()),
            'switching_profile_id': type.OptionalType(type.StringType()),
            'transport_node_id': type.OptionalType(type.StringType()),
            'transport_zone_id': type.OptionalType(type.StringType()),
        })
        getall_error_dict = {
            'com.vmware.vapi.std.errors.invalid_request':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'InvalidRequest'),
            'com.vmware.vapi.std.errors.unauthorized':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'Unauthorized'),
            'com.vmware.vapi.std.errors.service_unavailable':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'ServiceUnavailable'),
            'com.vmware.vapi.std.errors.internal_server_error':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'InternalServerError'),
            'com.vmware.vapi.std.errors.not_found':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'NotFound'),

        }
        getall_input_value_validator_list = [
        ]
        getall_output_validator_list = [
        ]
        getall_rest_metadata = OperationRestMetadata(
            http_method='GET',
            url_template='/api/v1/logical-ports/status',
            path_variables={
            },
             header_parameters={
                                             },
            query_parameters={
                'attachment_id': 'attachment_id',
                'attachment_type': 'attachment_type',
                'bridge_cluster_id': 'bridge_cluster_id',
                'container_ports_only': 'container_ports_only',
                'cursor': 'cursor',
                'diagnostic': 'diagnostic',
                'included_fields': 'included_fields',
                'logical_switch_id': 'logical_switch_id',
                'page_size': 'page_size',
                'parent_vif_id': 'parent_vif_id',
                'sort_ascending': 'sort_ascending',
                'sort_by': 'sort_by',
                'source': 'source',
                'switching_profile_id': 'switching_profile_id',
                'transport_node_id': 'transport_node_id',
                'transport_zone_id': 'transport_zone_id',
            },
            content_type='application/json'
        )

        operations = {
            'get': {
                'input_type': get_input_type,
                'output_type': type.ReferenceType('com.vmware.nsx.model_client', 'LogicalPortOperationalStatus'),
                'errors': get_error_dict,
                'input_value_validator_list': get_input_value_validator_list,
                'output_validator_list': get_output_validator_list,
                'task_type': TaskType.NONE,
            },
            'getall': {
                'input_type': getall_input_type,
                'output_type': type.ReferenceType('com.vmware.nsx.model_client', 'LogicalPortStatusSummary'),
                'errors': getall_error_dict,
                'input_value_validator_list': getall_input_value_validator_list,
                'output_validator_list': getall_output_validator_list,
                'task_type': TaskType.NONE,
            },
        }
        rest_metadata = {
            'get': get_rest_metadata,
            'getall': getall_rest_metadata,
        }
        ApiInterfaceStub.__init__(
            self, iface_name='com.vmware.nsx.logical_ports.status',
            config=config, operations=operations, rest_metadata=rest_metadata,
            is_vapi_rest=False)


class StubFactory(StubFactoryBase):
    _attrs = {
        'ForwardingPath': ForwardingPath,
        'MacTable': MacTable,
        'State': State,
        'Statistics': Statistics,
        'Status': Status,
    }




"""
DataDefinition classes
"""

__author__ = 'VMware, Inc.'
__copyright__ = 'Copyright 2015-2018 VMware, Inc.  All rights reserved. -- VMware Confidential'  # pylint: disable=line-too-long


from vmware.vapi.data.type import Type
from vmware.vapi.data.value import data_value_factory, StructValue
from vmware.vapi.exception import CoreException
from vmware.vapi.l10n.runtime import message_factory
from vmware.vapi.lib.constants import MAP_ENTRY, OPERATION_INPUT
from vmware.vapi.lib.log import get_vapi_logger
from vmware.vapi.lib.visitor import VapiVisitor

logger = get_vapi_logger(__name__)


class DataDefinition(object):
    """
    Base class for all types in the vAPI run-time type system

    :type type: :class:`str`
    :ivar type: String representation of the type
    """
    def __init__(self, data_type=None):
        """
        Initialize the data definition instance

        :type  data_type: :class:`str`
        :param data_type: String representation of the type
        """
        self.type = data_type

    def valid_instance_of(self, value):
        """
        Validates that the specified :class:`vmware.vapi.data.value.DataValue`
        is an instance of this data-definition

        :type  value: :class:`vmware.vapi.data.value.DataValue`
        :param value: the data value to validate

        :rtype: :class:`bool`
        :return: true if the value is an instance of this data-definition,
                 false, otherwise
        """
        return not self.validate(value)

    def validate(self, value):
        """
        Validates that the specified :class:`vmware.vapi.data.value.DataValue`
        is an instance of this data-definition

        :type  value: :class:`vmware.vapi.data.value.DataValue`
        :param value: the data value to validate

        :rtype: :class:`list` of :class:`vmware.vapi.message.Message`
                or ``None``
        :return: a stack of messages indicating why the value is not an instance
                 of this data-definition, or None if the value is an instance of
                 this data-definition
        """
        if value is None:
            msg = message_factory.get_message('vapi.data.validate.mismatch',
                                              self.type,
                                              'None')
            return [msg]

        if value.type != self.type:
            msg = message_factory.get_message('vapi.data.validate.mismatch',
                                              self.type,
                                              value.type)
            return [msg]

        return None

    def complete_value(self, value):
        """
        Fill the optional fields of StructValues. Also
        includes the StructValues present in other generic types: List
        and Optional.

        :type  value: :class:`vmware.vapi.data.value.DataValue`
        :param value: DataValue
        """
        return None

    def accept(self, visitor):
        """
        Applies a visitor to this data-definition.

        :type  visitor: :class:`SimpleDefinitionVisitor`
        :param visitor: the visitor operating on this data-definition
        """
        visitor.visit(self)

    def __eq__(self, other):
        if not isinstance(other, DataDefinition):
            return NotImplemented

        return self.type == other.type

    def __ne__(self, other):
        if not isinstance(other, DataDefinition):
            return NotImplemented
        return not (self == other)

    # Classes that override __eq__ should define __hash__ to make
    # its instances usable in hashed collections
    def __hash__(self):
        return str(self).__hash__()

    def __repr__(self):
        return '%s()' % self.__class__.__name__


class SingletonDefinition(DataDefinition):
    """
    Base class for all the primitive data definition classes. All the derived
    classes of this class will have only one instance.
    """
    _instance = None

    def __init__(self, data_type):
        """
        Initialize SingletonDefinition

        :type  data_type: :class:`vmware.vapi.data.type.Type`
        :param data_type: Type of the DataDefinition
        """
        DataDefinition.__init__(self, data_type)

    def __new__(cls, *args, **kwargs):
        if not cls._instance:
            cls._instance = super(SingletonDefinition, cls).__new__(
                cls, *args, **kwargs)
        return cls._instance


class VoidDefinition(SingletonDefinition):
    """
    DataDefinition class for void
    """
    def __init__(self):
        """
        Initialize VoidDefinition
        """
        SingletonDefinition.__init__(self, Type.VOID)

    def new_value(self):
        """
        Create a new VoidValue

        :rtype: :class:`vmware.vapi.data.value.VoidValue`
        :return: Newly created VoidValue
        """
        return data_value_factory(self.type)


class IntegerDefinition(SingletonDefinition):
    """
    DataDefinition for long values
    """
    def __init__(self):
        """
        Initialize IntegerDefinition
        """
        SingletonDefinition.__init__(self, Type.INTEGER)

    def new_value(self, value=0):
        """
        Create a new IntegerValue

        :rtype: :class:`vmware.vapi.data.value.IntegerValue`
        :return: Newly created IntegerValue
        """
        return data_value_factory(self.type, value)


class DoubleDefinition(SingletonDefinition):
    """
    DataDefinition for floats
    """
    def __init__(self):
        """
        Initialize DoubleDefinition
        """
        SingletonDefinition.__init__(self, Type.DOUBLE)

    def new_value(self, value=0.0):
        """
        Create a new DoubleValue

        :rtype: :class:`vmware.vapi.data.value.DoubleValue`
        :return: Newly created DoubleValue
        """
        return data_value_factory(self.type, value)


class StringDefinition(SingletonDefinition):
    """
    DataDefinition for strings
    """
    def __init__(self):
        """
        Initialize StringDefinition
        """
        SingletonDefinition.__init__(self, Type.STRING)

    def new_value(self, value=""):
        """
        Create a new StringValue

        :rtype: :class:`vmware.vapi.data.value.StringValue`
        :return: Newly created StringValue
        """
        return data_value_factory(self.type, value)


class OpaqueDefinition(SingletonDefinition):
    """
    DataDefinition for opaque
    """
    def __init__(self):
        """
        Initialize OpaqueDefinition
        """
        SingletonDefinition.__init__(self, Type.OPAQUE)

    def validate(self, value):
        """
        The validation for OpaqueDefinition will succeed against
        any DataValue. Only validates that supplied value is not None

        :type  value: :class:`vmware.vapi.data.value.DataValue`
        :param value: DataValue to be validated
        :rtype: :class:`list` of :class:`vmware.vapi.message.Message`
                 or ``None``
        :return: a stack of messages indicating why the value is not an instance
                 of this data-definition, or None if the value is an instance of
                 this data-definition
        """
        if value is None:
            msg = message_factory.get_message('vapi.data.validate.mismatch',
                                              self.type,
                                              'None')
            return [msg]
        return None


class DynamicStructDefinition(SingletonDefinition):
    """
    DataDefinition for dynamic structs
    """
    _valid_types = [Type.STRUCTURE, Type.ERROR]

    def __init__(self):
        """
        Initialize DynamicStructDefinition
        """
        SingletonDefinition.__init__(self, Type.DYNAMIC_STRUCTURE)

    def validate(self, value):
        """
        The validation for DynamicStructDefinition will succeed against
        any StructValue.

        :type  value: :class:`vmware.vapi.data.value.DataValue`
        :param value: the data value to validate

        :rtype: :class:`list` of :class:`vmware.vapi.message.Message`
                or ``None``
        :return: a stack of messages indicating why the value is not an instance
                 of this data-definition, or None if the value is an instance of
                 this data-definition
        """
        if value is None:
            msg = message_factory.get_message(
                'vapi.data.dynamicstruct.validate.mismatch',
                self._valid_types,
                'None')
            return [msg]

        if value.type not in self._valid_types:
            msg = message_factory.get_message(
                'vapi.data.dynamicstruct.validate.mismatch',
                self._valid_types,
                value.type)
            return [msg]

        return None


class AnyErrorDefinition(SingletonDefinition):
    """
    DataDefinition for 'Exception' type in IDL
    """
    def __init__(self):
        """
        Initialize AnyErrorDefinition
        """
        SingletonDefinition.__init__(self, Type.ANY_ERROR)

    def validate(self, value):
        """
        The validation for AnyErrorDefinition will succeed against
        any ErrorValue.

        :type  value: :class:`vmware.vapi.data.value.DataValue`
        :param value: the data value to validate

        :rtype: :class:`list` of :class:`vmware.vapi.message.Message`
                or ``None``
        :return: a stack of messages indicating why the value is not an instance
                 of this data-definition, or None if the value is an instance of
                 this data-definition
        """
        if value is None:
            msg = message_factory.get_message('vapi.data.validate.mismatch',
                                              self.type,
                                              'None')
            return [msg]

        if value.type != Type.ERROR:
            msg = message_factory.get_message('vapi.data.validate.mismatch',
                                              self.type,
                                              value.type)
            return [msg]

        return None


class BlobDefinition(SingletonDefinition):
    """
    DataDefinition for binary values
    """
    def __init__(self):
        """
        Initialize BlobDefinition
        """
        SingletonDefinition.__init__(self, Type.BLOB)

    def new_value(self, value=b''):
        """
        Create a new BlobValue

        :rtype: :class:`vmware.vapi.data.value.BlobValue`
        :return: Newly created BlobValue
        """
        return data_value_factory(self.type, value)


class BooleanDefinition(SingletonDefinition):
    """
    DataDefinition for bool values
    """
    def __init__(self):
        """
        Initialize BooleanDefinition
        """
        SingletonDefinition.__init__(self, Type.BOOLEAN)

    def new_value(self, value=False):
        """
        Create a new BooleanValue

        :rtype: :class:`vmware.vapi.data.value.BooleanValue`
        :return: Newly created BooleanValue
        """
        return data_value_factory(self.type, value)


class ListDefinition(DataDefinition):
    """
    DataDefinition for lists
    """
    def __init__(self, element_type):
        """
        Initialize ListDefinition

        :type  element_type: :class:`DataDefinition`
        :param element_type: DataDefinition of the elements inside
                ListDefinition
        """
        if not element_type:
            raise ValueError('ListDefinition requires element definition')
        DataDefinition.__init__(self, Type.LIST)
        self.element_type = element_type

    def new_value(self, values=None):
        """
        Create a new ListValue

        :type  values: :class:`list` of
                :class:`vmware.vapi.data.value.DataValue`
        :param values: List of elements
        :rtype: :class:`vmware.vapi.data.value.ListValue`
        :return: Newly created ListValue
        """
        return data_value_factory(self.type, values)

    def validate(self, list_value):
        """
        Apart from the validation checks specified in the validate
        method of DataDefinition class, this method does some additional checks

        Validation will fail if any element in the ListValue does not validate
        against the DataDefinition of the elementType of this ListDefinition

        :type  other: :class:`vmware.vapi.data.value.ListValue`
        :param other: ListValue to be validated
        :rtype: :class:`list` of :class:`vmware.vapi.message.Message`
                or ``None``
        :return: a stack of messages indicating why the value is not an instance
                 of this data-definition, or None if the value is an instance of
                 this data-definition
        """
        if isinstance(list_value, StructValue):
            return None

        errors = DataDefinition.validate(self, list_value)
        if errors:
            return errors

        for index, value in enumerate(list_value):
            errors = self.element_type.validate(value)
            if errors:
                msg = message_factory.get_message(
                    'vapi.data.list.invalid.entry',
                    str(value), index)
                return [msg] + errors

        return None

    def complete_value(self, value):
        """
        Fill the optional values inside StructValues

        :type  value: :class:`vmware.vapi.data.value.DataValue`
        :param value: DataValue
        """
        if value.type == Type.STRUCTURE:
            return None
        assert(value.type == Type.LIST)

        for element in value:
            self.element_type.complete_value(element)
        return None

    def __eq__(self, other):
        if not isinstance(other, ListDefinition):
            return NotImplemented

        return (DataDefinition.__eq__(self, other)
                and self.element_type == other.element_type)

    # Classes that override __eq__ should define __hash__ to make
    # its instances usable in hashed collections
    def __hash__(self):
        return DataDefinition.__hash__(self)

    def __repr__(self):
        return 'ListDefinition(element_type=%s)' % repr(self.element_type)


class StructDefinition(DataDefinition):
    """
    DataDefinition for structures
    """
    def __init__(self, name, fields, data_type=Type.STRUCTURE):
        """
        Initialize StructDefinition

        :type  name: :class:`str`
        :param name: Name of the Structure
        :type  fields: :class:`tuple` of (:class:`str`, :class:`DataDefinition`)
        :param fields: A tuple consisting of the field name and the field
                       definition for all the fields inside this
                       StructDefinition
        """
        if not name:
            raise ValueError('Struct name may not be None or empty string')
        DataDefinition.__init__(self, data_type)
        self.name = name
        self._keys = [k for (k, _) in fields]
        self._dict = dict(fields)

    def new_value(self):
        """
        Create a new StructValue

        :rtype: :class:`vmware.vapi.data.value.StructValue`
        :return: Newly created StructValue
        """
        return data_value_factory(self.type, self.name)

    def get_field_names(self):
        """
        Returns the list of field names in this struct definition. The ordering
        of fields is not preserved.

        :rtype: :class:`list` of :class:`str`
        :return: List of field names in this struct definition
        """
        return self._keys

    def get_field(self, field):
        """
        Returns the field definition of the specified field

        :rtype: :class:`DataDefinition`
        :return: field definition of the specified field
        """
        return self._dict.get(field)

    def validate(self, other):
        """
        Apart from the validation checks specified in the validate
        method of DataDefinition class, this method does some additional checks

        Validation will fail if
        - the name of the input StructValue does not match the name of this
        StructDefinition.
        - any of the fields (either required or optional) in this
        StructDefinition are not present in the input StructValue
        - the validation fails for any field value in the input StructValue with
        its corresponding definition in this StructDefinition

        The method performs inclusive validation. i.e. If there are any extra
        fields in the input StructValue which are not present in the
        StructDefinition, validation will not fail.

        :type  other: :class:`vmware.vapi.data.value.StructValue`
        :param other: StructValue to be validated
        :rtype: :class:`list` of :class:`vmware.vapi.message.Message`
                or ``None``
        :return: a stack of messages indicating why the value is not an instance
                 of this data-definition, or None if the value is an instance of
                 this data-definition
        """
        errors = DataDefinition.validate(self, other)
        if errors:
            return errors

        if (other.name != self.name):
            # Remove the following if condition in R28
            # To handle old client talking to new server
            if self.name in [MAP_ENTRY, OPERATION_INPUT]:
                pass
            else:
                msg = message_factory.get_message(
                    'vapi.data.structure.name.mismatch',
                    self.name, other.name)
                logger.debug(msg)
                return [msg]

        for field in self._keys:
            if (not other.has_field(field)
                    and self._dict.get(field).type != Type.OPTIONAL):
                msg = message_factory.get_message(
                    'vapi.data.structure.field.missing',
                    self.name, field)
                logger.debug(msg)
                return [msg]

            field_def = self._dict.get(field)
            field_val = other.get_field(field)
            errors = field_def.validate(field_val)
            if errors:
                msg = message_factory.get_message(
                    'vapi.data.structure.field.invalid',
                    field, self.name)
                logger.debug(msg)
                return [msg] + errors

        return None

    def complete_value(self, value):
        """
        Fill out all the unset optional fields in a structure
        based on the StructDefinition

        :type  value: :class:`vmware.vapi.data.value.StructValue`
        :param value: Input Struct Value
        """
        assert(value.type == self.type)
        assert(value is not None)

        for field in self._keys:
            field_def = self.get_field(field)
            if value.has_field(field):
                # Complete the optional values in elements as well
                field_def.complete_value(value.get_field(field))
            elif field_def.type == Type.OPTIONAL:
                # If field is not present and it is optional, fill it
                value.set_field(field, field_def.new_value())

    def __eq__(self, other):
        if not isinstance(other, StructDefinition):
            return NotImplemented

        if not (DataDefinition.__eq__(self, other) and other.name == self.name):
            return False

        for field in self._keys:
            other_field_def = other.get_field(field)
            if not other_field_def:
                return False
            field_def = self.get_field(field)
            if field_def != other_field_def:
                return False

        return True

    # Classes that override __eq__ should define __hash__ to make
    # its instances usable in hashed collections
    def __hash__(self):
        return DataDefinition.__hash__(self)

    def __repr__(self):
        fields = ", ".join(["%s=%s" % (field, repr(self._dict.get(field)))
                           for field in sorted(self._keys)])
        return '%s(name=%s, fields={%s})' % (self.__class__.__name__,
                                             repr(self.name),
                                             fields)


class StructRefDefinition(DataDefinition):
    """
    Reference to a StructDefinition. If the reference is resolved, it is
    bound to a specific StructDefinition target. If the reference is
    unresolved, its target is None.

    :type name: :class:`str`
    :ivar name: Structure name
    """
    def __init__(self, name, definition=None):
        """
        Initialize StructRefDefinition

        :type  name: :class:`str`
        :param name: Structure name
        :type  definition: :class:`DataDefinition`
        :param definition: If definition is passed, it creates a resolved
                           reference
        """
        if not name:
            raise ValueError('Struct name may not be None or empty string')
        self.name = name
        self._definition = definition
        DataDefinition.__init__(self, Type.STRUCTURE_REF)

    @property
    def target(self):
        """
        Returns the target structure definition of this reference.

        :rtype: :class:`DataDefinition` or ``None``
        :return: The target of this reference. The value will be None for
                 unresolved reference
        """
        return self._definition

    @target.setter
    def target(self, definition):
        """
        Resolves the reference. An unresolved reference can be resolved exactly
        once. A resolved reference cannot be re-resolved.

        :type  definition: :class:`DataDefinition`
        :param definition: structure definition
        :raise: :class:`vmware.vapi.exception.CoreException`: if the reference
            is already resolved (already has) a target) or if the name of the
            reference does not match the name of the definition
        """
        if not definition:
            raise ValueError('StructDefinition may not be None')
        if self._definition:
            msg = message_factory.get_message(
                'vapi.data.structref.already.resolved', self.name)
            logger.debug(msg)
            raise CoreException(msg)
        if self.name != definition.name:
            msg = message_factory.get_message(
                'vapi.data.structref.resolve.type.mismatch',
                self.name, definition.name)
            logger.debug(msg)
            raise CoreException(msg)
        self._definition = definition

    def check_resolved(self):
        """
        Check if the reference is resolved or not
        """
        if self._definition is None:
            msg = message_factory.get_message(
                'vapi.data.structref.not.resolved',
                self.name)
            logger.debug(msg)
            raise CoreException(msg)

    def complete_value(self, value):
        """
        Fill out all the unset optional fields in a structure
        based on the resolved StructDefinition for this
        StructRefDefinition.

        :type  value: :class:`vmware.vapi.data.value.StructValue`
        :param value: Input StructValue
        """
        self.check_resolved()
        self._definition.complete_value(value)

    def validate(self, other):
        """
        Validate using the target if the reference is resolved

        :type  other: :class:`vmware.vapi.data.value.StructValue`
        :param other: StructValue to be validated
        :rtype: :class:`list` of :class:`vmware.vapi.message.Message` of
                ``None``
        :return: a stack of messages indicating that the reference is not
                 resolved or why the value is not an instance of this
                 data-definition, or None if the value is an instance of
                 this data-definition
        """
        try:
            self.check_resolved()
        except CoreException as err:
            return err.messages
        return self._definition.validate(other)

    def __eq__(self, other):
        if not isinstance(other, StructRefDefinition):
            return NotImplemented

        if not (DataDefinition.__eq__(self, other)
                and other.name == self.name):
            return False

        return True

    # Classes that override __eq__ should define __hash__ to make
    # its instances usable in hashed collections
    def __hash__(self):
        return DataDefinition.__hash__(self)

    def __repr__(self):
        return 'StructRefDefinition(name=%s)' % self.name


class ErrorDefinition(StructDefinition):
    """
    DataDefinition for errors
    """
    def __init__(self, name, fields):
        """
        Initialize ErrorDefinition

        :type  name: :class:`str`
        :param name: Name of the Error
        :type  fields: :class:`tuple` of (:class:`str`, :class:`DataDefinition`)
        :param fields: A tuple consisting of the field name and the field
                       definition for all the fields inside this
                       StructDefinition
        """
        StructDefinition.__init__(self, name, fields, Type.ERROR)

    def new_value(self):
        """
        Create a new ErrorValue

        :rtype: :class:`vmware.vapi.data.value.ErrorValue`
        :return: Newly created ErrorValue
        """
        return data_value_factory(self.type, self.name)


class OptionalDefinition(DataDefinition):
    """
    An OptionalDefinition instance defines an optional type with a specified
    element type

    :type data_type: :class:`str`
    :ivar type: String representation of the type
    :type element_type: :class:`DataDefinition`
    :ivar element_type: The type of the element that is optional
    """
    def __init__(self, element_type):
        """
        Initialize OptionalDefinition

        :type  element_type: :class:`vmware.vapi.data.type.Type`
        :param element_type: Type of the DataDefinition
        """
        if not element_type:
            raise ValueError('Optional definition requires element definition')
        DataDefinition.__init__(self, Type.OPTIONAL)
        self.element_type = element_type

    def new_value(self, value=None):
        """
        Create and return a new :class:`OptionalValue` using this
        optional-definition.

        :type  value: :class:`vmware.vapi.data.value.DataValue`
        :param value: The element value

        :rtype: :class:`vmware.vapi.data.value.OptionalValue`
        :return: A new optional value using the given data-definition
        """
        return data_value_factory(self.type, value)

    def validate(self, value):
        """
        Apart from the validation checks specified in the validate
        method of DataDefinition class, this method does some additional checks.

        Since this is an OptionalValue, validation will succeed if the
        element value is None. If the element value is not None, validation
        will fail if element in the OptionalValue does not validate against
        the DataDefinition of the element_type of this OptionalDefinition.

        :type  other: :class:`vmware.vapi.data.value.OptionalValue`
        :param other: OptionalValue to be validated
        :rtype: :class:`list` of :class:`vmware.vapi.message.Message`
        :return: a stack of messages indicating why the value is not an instance
                 of this data-definition, or None if the value is an instance of
                 this data-definition
        """
        errors = DataDefinition.validate(self, value)
        if errors:
            return errors

        if value.value is None:
            # None is a valid value for optional
            pass
        elif value.is_set():
            errors = self.element_type.validate(value.value)
            if errors:
                msg = message_factory.get_message('vapi.data.optional.validate')
                return [msg] + errors

        return None

    def complete_value(self, value):
        """
        Fill the optional values inside StructValues

        :type  value: :class:`vmware.vapi.data.value.OptionalValue`
        :param value: DataValue
        """
        assert(value.type == Type.OPTIONAL)
        if value.is_set():
            self.element_type.complete_value(value.value)

    def __eq__(self, other):
        if not isinstance(other, OptionalDefinition):
            return NotImplemented

        return (DataDefinition.__eq__(self, other)
                and self.element_type == other.element_type)

    # Classes that override __eq__ should define __hash__ to make
    # its instances usable in hashed collections
    def __hash__(self):
        return DataDefinition.__hash__(self)

    def __repr__(self):
        return "OptionalDefinition(element_type=%s)" % repr(self.element_type)


class SecretDefinition(SingletonDefinition):
    """
    DataDefinition for Secrets. Only strings are allowed to be secrets.
    """
    def __init__(self):
        """
        Initialize SecretDefinition
        """
        SingletonDefinition.__init__(self, Type.SECRET)

    def new_value(self, value=""):
        """
        Create a new SecretValue

        :rtype: :class:`vmware.vapi.data.value.SecretValue`
        :return: Newly created SecretValue
        """
        return data_value_factory(self.type, value)


class SimpleDefinitionVisitor(VapiVisitor):
    """
    Base no-op implementation of a definition visitor
    """
    def __init__(self):
        VapiVisitor.__init__(self, 'Definition')

    def visit_void(self, defn):
        """
        Visit a VoidDefinition

        :type  defn: :class:`VoidDefinition`
        :param defn: Data definition
        """
        raise NotImplementedError

    def visit_integer(self, defn):
        """
        Visit a IntegerDefinition

        :type  defn: :class:`IntegerDefinition`
        :param defn: Data definition
        """
        raise NotImplementedError

    def visit_double(self, defn):
        """
        Visit a DoubleDefinition

        :type  defn: :class:`DoubleDefinition`
        :param defn: Data definition
        """
        raise NotImplementedError

    def visit_string(self, defn):
        """
        Visit a StringDefinition

        :type  defn: :class:`StringDefinition`
        :param defn: Data definition
        """
        raise NotImplementedError

    def visit_opaque(self, defn):
        """
        Visit a OpaqueDefinition

        :type  defn: :class:`OpaqueDefinition`
        :param defn: Data definition
        """
        raise NotImplementedError

    def visit_blob(self, defn):
        """
        Visit a BlobDefinition

        :type  defn: :class:`BlobDefinition`
        :param defn: Data definition
        """
        raise NotImplementedError

    def visit_boolean(self, defn):
        """
        Visit a BooleanDefinition

        :type  defn: :class:`BooleanDefinition`
        :param defn: Data definition
        """
        raise NotImplementedError

    def visit_list(self, defn):
        """
        Visit a ListDefinition

        :type  defn: :class:`ListDefinition`
        :param defn: Data definition
        """
        raise NotImplementedError

    def visit_struct(self, defn):
        """
        Visit a StructDefinition

        :type  defn: :class:`StructDefinition`
        :param defn: Data definition
        """
        raise NotImplementedError

    def visit_error(self, defn):
        """
        Visit an ErrorDefinition

        :type  defn: :class:`ErrorDefinition`
        :param defn: Data definition
        """
        raise NotImplementedError

    def visit_optional(self, defn):
        """
        Visit a OptionalDefinition

        :type  defn: :class:`OptionalDefinition`
        :param defn: Data definition
        """
        raise NotImplementedError

    def visit_secret(self, defn):
        """
        Visit a SecretDefinition

        :type  defn: :class:`SecretDefinition`
        :param defn: Data definition
        """
        raise NotImplementedError

    def visit_struct_ref(self, defn):
        """
        Visit a StructRefDefinition

        :type  defn: :class:`StructRefDefinition`
        :param defn: StructRefDefinition object
        """
        raise NotImplementedError


# vapi type name to vapi type definition map
data_type_to_definition_map = {
    Type.INTEGER: IntegerDefinition,
    Type.DOUBLE: DoubleDefinition,
    Type.BOOLEAN: BooleanDefinition,
    Type.STRING: StringDefinition,
    Type.BLOB: BlobDefinition,
    Type.LIST: ListDefinition,
    Type.STRUCTURE: StructDefinition,
    Type.OPTIONAL: OptionalDefinition,
    Type.VOID: VoidDefinition,
    Type.OPAQUE: OpaqueDefinition,
    Type.SECRET: SecretDefinition,
    Type.ERROR: ErrorDefinition,
    Type.STRUCTURE_REF: StructRefDefinition,
    Type.DYNAMIC_STRUCTURE: DynamicStructDefinition,
}


def data_definition_factory(data_type, *args, **kwargs):
    """
    data definition factory

    :type  data_type: :class:`str`
    :param data_type: Type name defined in vmware.vapi.data.type
    :type  args: :class:`set`
    :param args: Positional arguments to data definition constructor
    :type  kwargs: :class:`dict`
    :param kwargs: Dict arguments to data definition constructor
    """
    constructor = data_type_to_definition_map.get(data_type)
    return constructor(*args, **kwargs)


class ReferenceResolver(object):
    """
    Resolves all the StructRefDefinition objects
    """
    def __init__(self):
        """
        Initialize ReferenceResolver
        """
        self._definitions = {}
        self._references = []

    def add_definition(self, definition):
        """
        Adds a new structure definition to the context

        :type  definition: :class:`StructDefinition`
        :param definition: StructDefinition
        """
        self._definitions[definition.name] = definition

    def add_reference(self, reference):
        """
        Adds a new structure refernce to the context

        :type  refernce: :class:`StructRefDefinition`
        :param refernce: StructRefDefinition
        """
        self._references.append(reference)

    def is_defined(self, name):
        """
        Determines whether the context contains a definition for the specified
        structure

        :rtype: :class:`bool`
        :return: True if the structure is already defined, false otherwise
        """
        return name in self._definitions

    def get_definition(self, name):
        """
        Determines whether the context contains a definition for the specified
        structure

        :rtype: :class:`StructDefinition`
        :return: Definition if the structure if it is already defined,
                 None otherwise
        """
        if self.is_defined(name):
            return self._definitions[name]
        return None

    def resolve(self):
        """
        Traverses all references and resolves the unresolved ones.
        """
        for reference in self._references:
            if reference.target is None:
                definition = self._definitions.get(reference.name)
                if definition is None:
                    msg = message_factory.get_message(
                        'vapi.data.structref.structure.not.defined',
                        reference.name)
                    logger.debug(msg)
                    raise CoreException(msg)
                reference.target = definition



# -*- coding: utf-8 -*-
#---------------------------------------------------------------------------
# Copyright 2023 VMware, Inc.  All rights reserved.

# AUTO GENERATED FILE -- DO NOT MODIFY!
#
# vAPI stub file for package com.vmware.nsx.logical_routers.
#---------------------------------------------------------------------------

"""


"""

__author__ = 'VMware, Inc.'
__docformat__ = 'restructuredtext en'

import sys
from warnings import warn

from vmware.vapi.bindings import type
from vmware.vapi.bindings.converter import TypeConverter
from vmware.vapi.bindings.enum import Enum
from vmware.vapi.bindings.error import VapiError
from vmware.vapi.bindings.struct import VapiStruct
from vmware.vapi.bindings.stub import (
    ApiInterfaceStub, StubFactoryBase, VapiInterface)
from vmware.vapi.bindings.common import raise_core_exception
from vmware.vapi.data.validator import (UnionValidator, HasFieldsOfValidator)
from vmware.vapi.exception import CoreException
from vmware.vapi.lib.constants import TaskType
from vmware.vapi.lib.rest import OperationRestMetadata


class Routing(VapiInterface):
    """
    
    """

    _VAPI_SERVICE_ID = 'com.vmware.nsx.logical_routers.routing'
    """
    Identifier of the service in canonical form.
    """
    def __init__(self, config):
        """
        :type  config: :class:`vmware.vapi.bindings.stub.StubConfiguration`
        :param config: Configuration to be used for creating the stub.
        """
        VapiInterface.__init__(self, config, _RoutingStub)
        self._VAPI_OPERATION_IDS = {}


    def get(self,
            logical_router_id,
            ):
        """
        Returns the routing configuration for a specified logical router. 
        
        Please use below Policy APIs. 
        GET /policy/api/v1/infra/tier-0s/<tier-0-id> 

        .. deprecated:: unknown

        :type  logical_router_id: :class:`str`
        :param logical_router_id: (required)
        :rtype: :class:`com.vmware.nsx.model_client.RoutingConfig`
        :return: com.vmware.nsx.model.RoutingConfig
        :raise: :class:`com.vmware.vapi.std.errors_client.InvalidRequest` 
            Bad Request, Precondition Failed
        :raise: :class:`com.vmware.vapi.std.errors_client.Unauthorized` 
            Forbidden
        :raise: :class:`com.vmware.vapi.std.errors_client.ServiceUnavailable` 
            Service Unavailable
        :raise: :class:`com.vmware.vapi.std.errors_client.InternalServerError` 
            Internal Server Error
        :raise: :class:`com.vmware.vapi.std.errors_client.NotFound` 
            Not Found
        """
        warn('com.vmware.nsx.logical_routers.Routing.get is deprecated.', DeprecationWarning)
        return self._invoke('get',
                            {
                            'logical_router_id': logical_router_id,
                            })

    def update(self,
               logical_router_id,
               routing_config,
               ):
        """
        Modifies the routing configuration for a specified logical router. 
        
        Please use below Policy APIs. 
        PUT /policy/api/v1/infra/tier-0s/<tier-0-id> 

        .. deprecated:: unknown

        :type  logical_router_id: :class:`str`
        :param logical_router_id: (required)
        :type  routing_config: :class:`com.vmware.nsx.model_client.RoutingConfig`
        :param routing_config: (required)
        :rtype: :class:`com.vmware.nsx.model_client.RoutingConfig`
        :return: com.vmware.nsx.model.RoutingConfig
        :raise: :class:`com.vmware.vapi.std.errors_client.InvalidRequest` 
            Bad Request, Precondition Failed
        :raise: :class:`com.vmware.vapi.std.errors_client.Unauthorized` 
            Forbidden
        :raise: :class:`com.vmware.vapi.std.errors_client.ServiceUnavailable` 
            Service Unavailable
        :raise: :class:`com.vmware.vapi.std.errors_client.InternalServerError` 
            Internal Server Error
        :raise: :class:`com.vmware.vapi.std.errors_client.NotFound` 
            Not Found
        """
        warn('com.vmware.nsx.logical_routers.Routing.update is deprecated.', DeprecationWarning)
        return self._invoke('update',
                            {
                            'logical_router_id': logical_router_id,
                            'routing_config': routing_config,
                            })
class State(VapiInterface):
    """
    
    """

    _VAPI_SERVICE_ID = 'com.vmware.nsx.logical_routers.state'
    """
    Identifier of the service in canonical form.
    """
    def __init__(self, config):
        """
        :type  config: :class:`vmware.vapi.bindings.stub.StubConfiguration`
        :param config: Configuration to be used for creating the stub.
        """
        VapiInterface.__init__(self, config, _StateStub)
        self._VAPI_OPERATION_IDS = {}


    def get(self,
            logical_router_id,
            barrier_id=None,
            request_id=None,
            ):
        """
        Return realized state information of a logical router. Any
        configuration update that affects the logical router can use this API
        to get its realized state by passing a request_id returned by the
        configuration change operation. e.g. Update configuration of logical
        router, static routes, etc. 
        
        Please use below policy apis instead of this API. 
        GET /infra/tier-0s/<tier-0-id>/state 
        GET /infra/tier-1s/<tier-1-id>/state 

        .. deprecated:: unknown

        :type  logical_router_id: :class:`str`
        :param logical_router_id: (required)
        :type  barrier_id: :class:`long` or ``None``
        :param barrier_id: (optional)
        :type  request_id: :class:`str` or ``None``
        :param request_id: Realization request ID (optional)
        :rtype: :class:`com.vmware.nsx.model_client.LogicalRouterState`
        :return: com.vmware.nsx.model.LogicalRouterState
        :raise: :class:`com.vmware.vapi.std.errors_client.InvalidRequest` 
            Bad Request, Precondition Failed
        :raise: :class:`com.vmware.vapi.std.errors_client.Unauthorized` 
            Forbidden
        :raise: :class:`com.vmware.vapi.std.errors_client.ServiceUnavailable` 
            Service Unavailable
        :raise: :class:`com.vmware.vapi.std.errors_client.InternalServerError` 
            Internal Server Error
        :raise: :class:`com.vmware.vapi.std.errors_client.NotFound` 
            Not Found
        """
        warn('com.vmware.nsx.logical_routers.State.get is deprecated.', DeprecationWarning)
        return self._invoke('get',
                            {
                            'logical_router_id': logical_router_id,
                            'barrier_id': barrier_id,
                            'request_id': request_id,
                            })
class Status(VapiInterface):
    """
    
    """
    GET_SOURCE_REALTIME = "realtime"
    """
    Possible value for ``source`` of method :func:`Status.get`.

    """
    GET_SOURCE_CACHED = "cached"
    """
    Possible value for ``source`` of method :func:`Status.get`.

    """

    _VAPI_SERVICE_ID = 'com.vmware.nsx.logical_routers.status'
    """
    Identifier of the service in canonical form.
    """
    def __init__(self, config):
        """
        :type  config: :class:`vmware.vapi.bindings.stub.StubConfiguration`
        :param config: Configuration to be used for creating the stub.
        """
        VapiInterface.__init__(self, config, _StatusStub)
        self._VAPI_OPERATION_IDS = {}


    def get(self,
            logical_router_id,
            source=None,
            ):
        """
        Returns status for the Logical Router of the given id. 
        
        Please use below Policy API. 
        GET /policy/api/v1/infra/tier-0s/<tier-0-id>/state 
        GET /policy/api/v1/infra/tier-1s/<tier-1-id>/state

        .. deprecated:: unknown

        :type  logical_router_id: :class:`str`
        :param logical_router_id: (required)
        :type  source: :class:`str` or ``None``
        :param source: Data source type. (optional)
        :rtype: :class:`com.vmware.nsx.model_client.LogicalRouterStatus`
        :return: com.vmware.nsx.model.LogicalRouterStatus
        :raise: :class:`com.vmware.vapi.std.errors_client.InvalidRequest` 
            Bad Request, Precondition Failed
        :raise: :class:`com.vmware.vapi.std.errors_client.Unauthorized` 
            Forbidden
        :raise: :class:`com.vmware.vapi.std.errors_client.ServiceUnavailable` 
            Service Unavailable
        :raise: :class:`com.vmware.vapi.std.errors_client.InternalServerError` 
            Internal Server Error
        :raise: :class:`com.vmware.vapi.std.errors_client.NotFound` 
            Not Found
        """
        warn('com.vmware.nsx.logical_routers.Status.get is deprecated.', DeprecationWarning)
        return self._invoke('get',
                            {
                            'logical_router_id': logical_router_id,
                            'source': source,
                            })
class _RoutingStub(ApiInterfaceStub):
    def __init__(self, config):
        # properties for get operation
        get_input_type = type.StructType('operation-input', {
            'logical_router_id': type.StringType(),
        })
        get_error_dict = {
            'com.vmware.vapi.std.errors.invalid_request':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'InvalidRequest'),
            'com.vmware.vapi.std.errors.unauthorized':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'Unauthorized'),
            'com.vmware.vapi.std.errors.service_unavailable':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'ServiceUnavailable'),
            'com.vmware.vapi.std.errors.internal_server_error':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'InternalServerError'),
            'com.vmware.vapi.std.errors.not_found':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'NotFound'),

        }
        get_input_value_validator_list = [
        ]
        get_output_validator_list = [
        ]
        get_rest_metadata = OperationRestMetadata(
            http_method='GET',
            url_template='/api/v1/logical-routers/{logical-router-id}/routing',
            path_variables={
                'logical_router_id': 'logical-router-id',
            },
             header_parameters={
               },
            query_parameters={
            },
            content_type='application/json'
        )

        # properties for update operation
        update_input_type = type.StructType('operation-input', {
            'logical_router_id': type.StringType(),
            'routing_config': type.ReferenceType('com.vmware.nsx.model_client', 'RoutingConfig'),
        })
        update_error_dict = {
            'com.vmware.vapi.std.errors.invalid_request':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'InvalidRequest'),
            'com.vmware.vapi.std.errors.unauthorized':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'Unauthorized'),
            'com.vmware.vapi.std.errors.service_unavailable':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'ServiceUnavailable'),
            'com.vmware.vapi.std.errors.internal_server_error':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'InternalServerError'),
            'com.vmware.vapi.std.errors.not_found':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'NotFound'),

        }
        update_input_value_validator_list = [
        ]
        update_output_validator_list = [
        ]
        update_rest_metadata = OperationRestMetadata(
            http_method='PUT',
            url_template='/api/v1/logical-routers/{logical-router-id}/routing',
            request_body_parameter='routing_config',
            path_variables={
                'logical_router_id': 'logical-router-id',
            },
             header_parameters={
                 },
            query_parameters={
            },
            content_type='application/json'
        )

        operations = {
            'get': {
                'input_type': get_input_type,
                'output_type': type.ReferenceType('com.vmware.nsx.model_client', 'RoutingConfig'),
                'errors': get_error_dict,
                'input_value_validator_list': get_input_value_validator_list,
                'output_validator_list': get_output_validator_list,
                'task_type': TaskType.NONE,
            },
            'update': {
                'input_type': update_input_type,
                'output_type': type.ReferenceType('com.vmware.nsx.model_client', 'RoutingConfig'),
                'errors': update_error_dict,
                'input_value_validator_list': update_input_value_validator_list,
                'output_validator_list': update_output_validator_list,
                'task_type': TaskType.NONE,
            },
        }
        rest_metadata = {
            'get': get_rest_metadata,
            'update': update_rest_metadata,
        }
        ApiInterfaceStub.__init__(
            self, iface_name='com.vmware.nsx.logical_routers.routing',
            config=config, operations=operations, rest_metadata=rest_metadata,
            is_vapi_rest=False)

class _StateStub(ApiInterfaceStub):
    def __init__(self, config):
        # properties for get operation
        get_input_type = type.StructType('operation-input', {
            'logical_router_id': type.StringType(),
            'barrier_id': type.OptionalType(type.IntegerType()),
            'request_id': type.OptionalType(type.StringType()),
        })
        get_error_dict = {
            'com.vmware.vapi.std.errors.invalid_request':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'InvalidRequest'),
            'com.vmware.vapi.std.errors.unauthorized':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'Unauthorized'),
            'com.vmware.vapi.std.errors.service_unavailable':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'ServiceUnavailable'),
            'com.vmware.vapi.std.errors.internal_server_error':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'InternalServerError'),
            'com.vmware.vapi.std.errors.not_found':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'NotFound'),

        }
        get_input_value_validator_list = [
        ]
        get_output_validator_list = [
        ]
        get_rest_metadata = OperationRestMetadata(
            http_method='GET',
            url_template='/api/v1/logical-routers/{logical-router-id}/state',
            path_variables={
                'logical_router_id': 'logical-router-id',
            },
             header_parameters={
                   },
            query_parameters={
                'barrier_id': 'barrier_id',
                'request_id': 'request_id',
            },
            content_type='application/json'
        )

        operations = {
            'get': {
                'input_type': get_input_type,
                'output_type': type.ReferenceType('com.vmware.nsx.model_client', 'LogicalRouterState'),
                'errors': get_error_dict,
                'input_value_validator_list': get_input_value_validator_list,
                'output_validator_list': get_output_validator_list,
                'task_type': TaskType.NONE,
            },
        }
        rest_metadata = {
            'get': get_rest_metadata,
        }
        ApiInterfaceStub.__init__(
            self, iface_name='com.vmware.nsx.logical_routers.state',
            config=config, operations=operations, rest_metadata=rest_metadata,
            is_vapi_rest=False)

class _StatusStub(ApiInterfaceStub):
    def __init__(self, config):
        # properties for get operation
        get_input_type = type.StructType('operation-input', {
            'logical_router_id': type.StringType(),
            'source': type.OptionalType(type.StringType()),
        })
        get_error_dict = {
            'com.vmware.vapi.std.errors.invalid_request':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'InvalidRequest'),
            'com.vmware.vapi.std.errors.unauthorized':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'Unauthorized'),
            'com.vmware.vapi.std.errors.service_unavailable':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'ServiceUnavailable'),
            'com.vmware.vapi.std.errors.internal_server_error':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'InternalServerError'),
            'com.vmware.vapi.std.errors.not_found':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'NotFound'),

        }
        get_input_value_validator_list = [
        ]
        get_output_validator_list = [
        ]
        get_rest_metadata = OperationRestMetadata(
            http_method='GET',
            url_template='/api/v1/logical-routers/{logical-router-id}/status',
            path_variables={
                'logical_router_id': 'logical-router-id',
            },
             header_parameters={
                 },
            query_parameters={
                'source': 'source',
            },
            content_type='application/json'
        )

        operations = {
            'get': {
                'input_type': get_input_type,
                'output_type': type.ReferenceType('com.vmware.nsx.model_client', 'LogicalRouterStatus'),
                'errors': get_error_dict,
                'input_value_validator_list': get_input_value_validator_list,
                'output_validator_list': get_output_validator_list,
                'task_type': TaskType.NONE,
            },
        }
        rest_metadata = {
            'get': get_rest_metadata,
        }
        ApiInterfaceStub.__init__(
            self, iface_name='com.vmware.nsx.logical_routers.status',
            config=config, operations=operations, rest_metadata=rest_metadata,
            is_vapi_rest=False)


class StubFactory(StubFactoryBase):
    _attrs = {
        'Routing': Routing,
        'State': State,
        'Status': Status,
        'nat': 'com.vmware.nsx.logical_routers.nat_client.StubFactory',
        'routing': 'com.vmware.nsx.logical_routers.routing_client.StubFactory',
        'service_cluster': 'com.vmware.nsx.logical_routers.service_cluster_client.StubFactory',
    }




# -*- coding: utf-8 -*-
#---------------------------------------------------------------------------
# Copyright 2023 VMware, Inc.  All rights reserved.

# AUTO GENERATED FILE -- DO NOT MODIFY!
#
# vAPI stub file for package com.vmware.nsx_policy.orgs.projects.infra.firewall.
#---------------------------------------------------------------------------

"""


"""

__author__ = 'VMware, Inc.'
__docformat__ = 'restructuredtext en'

import sys
from warnings import warn

from vmware.vapi.bindings import type
from vmware.vapi.bindings.converter import TypeConverter
from vmware.vapi.bindings.enum import Enum
from vmware.vapi.bindings.error import VapiError
from vmware.vapi.bindings.struct import VapiStruct
from vmware.vapi.bindings.stub import (
    ApiInterfaceStub, StubFactoryBase, VapiInterface)
from vmware.vapi.bindings.common import raise_core_exception
from vmware.vapi.data.validator import (UnionValidator, HasFieldsOfValidator)
from vmware.vapi.exception import CoreException
from vmware.vapi.lib.constants import TaskType
from vmware.vapi.lib.rest import OperationRestMetadata


class Policies(VapiInterface):
    """
    
    """

    _VAPI_SERVICE_ID = 'com.vmware.nsx_policy.orgs.projects.infra.firewall.policies'
    """
    Identifier of the service in canonical form.
    """
    def __init__(self, config):
        """
        :type  config: :class:`vmware.vapi.bindings.stub.StubConfiguration`
        :param config: Configuration to be used for creating the stub.
        """
        VapiInterface.__init__(self, config, _PoliciesStub)
        self._VAPI_OPERATION_IDS = {}


    def list(self,
             org_id,
             project_id,
             scope,
             enforcement_point_path=None,
             parent_path=None,
             ):
        """
        Get the list of policies filtered based on the given criteria.

        :type  org_id: :class:`str`
        :param org_id: The organization ID (required)
        :type  project_id: :class:`str`
        :param project_id: The project ID (required)
        :type  scope: :class:`str`
        :param scope: Scope filter criteria (required)
        :type  enforcement_point_path: :class:`str` or ``None``
        :param enforcement_point_path: Path of the enforcement point (optional)
        :type  parent_path: :class:`str` or ``None``
        :param parent_path: Path of the parent object of the entities (optional)
        :rtype: :class:`com.vmware.nsx_policy.model_client.PolicyResourceReferenceForEPListResult`
        :return: com.vmware.nsx_policy.model.PolicyResourceReferenceForEPListResult
        :raise: :class:`com.vmware.vapi.std.errors_client.InvalidRequest` 
            Bad Request, Precondition Failed
        :raise: :class:`com.vmware.vapi.std.errors_client.Unauthorized` 
            Forbidden
        :raise: :class:`com.vmware.vapi.std.errors_client.ServiceUnavailable` 
            Service Unavailable
        :raise: :class:`com.vmware.vapi.std.errors_client.InternalServerError` 
            Internal Server Error
        :raise: :class:`com.vmware.vapi.std.errors_client.NotFound` 
            Not Found
        """
        return self._invoke('list',
                            {
                            'org_id': org_id,
                            'project_id': project_id,
                            'scope': scope,
                            'enforcement_point_path': enforcement_point_path,
                            'parent_path': parent_path,
                            })
class Rules(VapiInterface):
    """
    
    """

    _VAPI_SERVICE_ID = 'com.vmware.nsx_policy.orgs.projects.infra.firewall.rules'
    """
    Identifier of the service in canonical form.
    """
    def __init__(self, config):
        """
        :type  config: :class:`vmware.vapi.bindings.stub.StubConfiguration`
        :param config: Configuration to be used for creating the stub.
        """
        VapiInterface.__init__(self, config, _RulesStub)
        self._VAPI_OPERATION_IDS = {}


    def list(self,
             org_id,
             project_id,
             scope,
             enforcement_point_path=None,
             parent_path=None,
             ):
        """
        Get the list of rules of given parent path of policy/section, filtered
        based on the given criteria. Parent path is mandatory.

        :type  org_id: :class:`str`
        :param org_id: The organization ID (required)
        :type  project_id: :class:`str`
        :param project_id: The project ID (required)
        :type  scope: :class:`str`
        :param scope: Scope filter criteria (required)
        :type  enforcement_point_path: :class:`str` or ``None``
        :param enforcement_point_path: Path of the enforcement point (optional)
        :type  parent_path: :class:`str` or ``None``
        :param parent_path: Path of the parent object of the entities (optional)
        :rtype: :class:`com.vmware.nsx_policy.model_client.PolicyResourceReferenceForEPListResult`
        :return: com.vmware.nsx_policy.model.PolicyResourceReferenceForEPListResult
        :raise: :class:`com.vmware.vapi.std.errors_client.InvalidRequest` 
            Bad Request, Precondition Failed
        :raise: :class:`com.vmware.vapi.std.errors_client.Unauthorized` 
            Forbidden
        :raise: :class:`com.vmware.vapi.std.errors_client.ServiceUnavailable` 
            Service Unavailable
        :raise: :class:`com.vmware.vapi.std.errors_client.InternalServerError` 
            Internal Server Error
        :raise: :class:`com.vmware.vapi.std.errors_client.NotFound` 
            Not Found
        """
        return self._invoke('list',
                            {
                            'org_id': org_id,
                            'project_id': project_id,
                            'scope': scope,
                            'enforcement_point_path': enforcement_point_path,
                            'parent_path': parent_path,
                            })
class _PoliciesStub(ApiInterfaceStub):
    def __init__(self, config):
        # properties for list operation
        list_input_type = type.StructType('operation-input', {
            'org_id': type.StringType(),
            'project_id': type.StringType(),
            'scope': type.StringType(),
            'enforcement_point_path': type.OptionalType(type.StringType()),
            'parent_path': type.OptionalType(type.StringType()),
        })
        list_error_dict = {
            'com.vmware.vapi.std.errors.invalid_request':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'InvalidRequest'),
            'com.vmware.vapi.std.errors.unauthorized':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'Unauthorized'),
            'com.vmware.vapi.std.errors.service_unavailable':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'ServiceUnavailable'),
            'com.vmware.vapi.std.errors.internal_server_error':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'InternalServerError'),
            'com.vmware.vapi.std.errors.not_found':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'NotFound'),

        }
        list_input_value_validator_list = [
        ]
        list_output_validator_list = [
        ]
        list_rest_metadata = OperationRestMetadata(
            http_method='GET',
            url_template='/policy/api/v1/orgs/{org-id}/projects/{project-id}/infra/firewall/policies',
            path_variables={
                'org_id': 'org-id',
                'project_id': 'project-id',
            },
             header_parameters={
                       },
            query_parameters={
                'scope': 'scope',
                'enforcement_point_path': 'enforcement_point_path',
                'parent_path': 'parent_path',
            },
            content_type='application/json'
        )

        operations = {
            'list': {
                'input_type': list_input_type,
                'output_type': type.ReferenceType('com.vmware.nsx_policy.model_client', 'PolicyResourceReferenceForEPListResult'),
                'errors': list_error_dict,
                'input_value_validator_list': list_input_value_validator_list,
                'output_validator_list': list_output_validator_list,
                'task_type': TaskType.NONE,
            },
        }
        rest_metadata = {
            'list': list_rest_metadata,
        }
        ApiInterfaceStub.__init__(
            self, iface_name='com.vmware.nsx_policy.orgs.projects.infra.firewall.policies',
            config=config, operations=operations, rest_metadata=rest_metadata,
            is_vapi_rest=False)

class _RulesStub(ApiInterfaceStub):
    def __init__(self, config):
        # properties for list operation
        list_input_type = type.StructType('operation-input', {
            'org_id': type.StringType(),
            'project_id': type.StringType(),
            'scope': type.StringType(),
            'enforcement_point_path': type.OptionalType(type.StringType()),
            'parent_path': type.OptionalType(type.StringType()),
        })
        list_error_dict = {
            'com.vmware.vapi.std.errors.invalid_request':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'InvalidRequest'),
            'com.vmware.vapi.std.errors.unauthorized':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'Unauthorized'),
            'com.vmware.vapi.std.errors.service_unavailable':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'ServiceUnavailable'),
            'com.vmware.vapi.std.errors.internal_server_error':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'InternalServerError'),
            'com.vmware.vapi.std.errors.not_found':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'NotFound'),

        }
        list_input_value_validator_list = [
        ]
        list_output_validator_list = [
        ]
        list_rest_metadata = OperationRestMetadata(
            http_method='GET',
            url_template='/policy/api/v1/orgs/{org-id}/projects/{project-id}/infra/firewall/rules',
            path_variables={
                'org_id': 'org-id',
                'project_id': 'project-id',
            },
             header_parameters={
                       },
            query_parameters={
                'scope': 'scope',
                'enforcement_point_path': 'enforcement_point_path',
                'parent_path': 'parent_path',
            },
            content_type='application/json'
        )

        operations = {
            'list': {
                'input_type': list_input_type,
                'output_type': type.ReferenceType('com.vmware.nsx_policy.model_client', 'PolicyResourceReferenceForEPListResult'),
                'errors': list_error_dict,
                'input_value_validator_list': list_input_value_validator_list,
                'output_validator_list': list_output_validator_list,
                'task_type': TaskType.NONE,
            },
        }
        rest_metadata = {
            'list': list_rest_metadata,
        }
        ApiInterfaceStub.__init__(
            self, iface_name='com.vmware.nsx_policy.orgs.projects.infra.firewall.rules',
            config=config, operations=operations, rest_metadata=rest_metadata,
            is_vapi_rest=False)


class StubFactory(StubFactoryBase):
    _attrs = {
        'Policies': Policies,
        'Rules': Rules,
    }

